
use <../../WebWeaver/scad/Util.scad>;
use <../../WebWeaver/scad/Material.scad>;

$fs = 0.35;
$fa = 2 * $fs;

$str = 3;
$E   = 0.01;  // ε

// note: using constant functions here because they get imported via `use`

// exporting *everything* as a constant function is bad.
// taking the ingenious concept from `Project 4` of the book from openscad-meistern.de

pcb_data = [

  ["pcb_t", 1.6],
  ["pcb_sx", 75.5],
  ["pcb_sy", 65.0],

  ["pcb_edge_r", 3.0],

  ["M3", 3.2],

  // LED terminal data
  ["tled_x", 5.75 + 23],
  ["tled_y", [-18.5, 0, 18.5]],

  // co = cut-out
  ["co_w", 6.1],
  ["co_h", 18.0],
  ["co_x", -34.7],
  ["co_y", -8.6],


  ["sc_left_off", 5.0],  // left side screw holes offset from edge


];

function PCB(key) = [ for (d=pcb_data) if (d[0] == key) d[1] ][0];





// x is all the same and y is +- 18.5 and 0 as there are three of them
function terminals_x() = 28.75;
function terminals_y() = 18.50;

// tpow = power terminal
function tpow_x()  = -30.25;
function tpow_y()  =  14.50;
// function tpow_sx() =  11.00;
// function tpow_sy() =  11.50;
// function tpow_sz() =   3.00;  // TODO: measure!!



module pcbScrewHoles(tz=0) {
  for (tx=[-32.75, 5.75], ty=[-27.5, 27.5]) 
    translate([tx, ty, tz])
      children();
}

module pcbOutline() {

  difference() {
    roundRect(PCB("pcb_sx"), PCB("pcb_sy"), PCB("pcb_edge_r"), center=true);
    
    pcbScrewHoles() circle(d=PCB("M3"));
    
    translate([PCB("co_x"), PCB("co_y"), 0]) square([PCB("co_w") + $E, PCB("co_h")], center=true);
  }
}

module pcb3D(pcb_t=PCB("pcb_t")) {
  PurpleFR4()
    linear_extrude(PCB("pcb_t"))
      pcbOutline();

  if ($preview) {

    pcbScrewHoles()
      Solder()
        difference() {
          translate([0, 0, -$E]) cylinder(d=PCB("M3")*2, h=PCB("pcb_t")+2*$E);
          translate([0, 0, -$E*2]) cylinder(d=PCB("M3")-$E,   h=PCB("pcb_t")+4*$E);
        }

//    Solder()
//      translate([tpow_x(), tpow_y(), PCB("pcb_t") - $E])
//        cube([tpow_sx(), tpow_sy(), tpow_sz()], center=true);
//    translate([tpow_x(), tpow_y(), PCB("pcb_t") - $E])
//      terminal2();


    translate([tpow_x(), tpow_y(), PCB("pcb_t")])
      terminal2();

    for (ty=PCB("tled_y"))
      translate([PCB("tled_x"), ty, PCB("pcb_t")])
        rotate([0, 0, 180])
          terminal3();

    for (ty=[-aj_y(), aj_y()-aj_sy()])
      translate([aj_x(), ty, PCB("pcb_t")])
        audioJack();

  }
}


// MP = measure precisely!

t_h0 =  9.5;  // TODO: MP!
t_h1 = 14.0;  // TODO: MP!
t_l_sx = 5.5;  // TODO: MP!  // l = lever
t_oh = 1.5;  // oh = overhang

t2_sx = 11.0 + t_oh;
t2_sy = 11.5;
t2_l_sy = 8.5;  // TODO: MP!
module terminal2() {
  GreyPlastic() difference() {
    union() {
      translate([-t_oh/2, 0, 0])
        linear_extrude(t_h0)
          roundRect(t2_sx, t2_sy, 0.69, center=true);

      translate([-t_oh/2, 0, 0])
        linear_extrude(t_h1)
          roundRect(t_l_sx, t2_l_sy, 1.337, center=true);
    }

    _ty = t2_sy * 1/4;
    for (ty=[-_ty, _ty])
      translate([-t2_sx/2, ty, t_h0/2])
        rotate([0, -90, 0])
          cylinder(d=3, h=8, center=true);
  }
}

t3_sx = 11.0 + t_oh;
t3_sy = 16.0;
t3_l_sy = 13.5;  // TODO: MP!
module terminal3() {
  GreyPlastic() difference() {
    union() {
      translate([-t_oh/2, 0, 0])
        linear_extrude(t_h0)
          roundRect(t3_sx, t3_sy, 0.69, center=true);

      translate([-t_oh/2, 0, 0])
        linear_extrude(t_h1)
          roundRect(t_l_sx, t3_l_sy, 1.337, center=true);
    }

    _ty = t3_sy * 1/3;
    for (ty=[-_ty, 0, _ty])
      translate([-t3_sx/2, ty, t_h0/2])
        rotate([0, -90, 0])
          cylinder(d=3, h=8, center=true);
  }
}


// aj = audio jack
function aj_x() = -12.25;
function aj_y() = PCB("pcb_sy")/2;
function aj_h() = 5.0;
function aj_sx() = 12.4;
function aj_sy() = 11.7;

module audioJack() {
  BlackPlastic()
    translate([-aj_sx()/2, 0, 0])
      cube([aj_sx(), aj_sy(), aj_h()]);
}


// audioJack();

pcb3D();
